package com.ycwl.basic.device.operator.impl;

import cn.hutool.core.date.DateUtil;
import com.aliyun.oss.HttpMethod;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.DeleteObjectsRequest;
import com.aliyun.oss.model.ListObjectsV2Request;
import com.aliyun.oss.model.ListObjectsV2Result;
import com.aliyun.oss.model.OSSObjectSummary;
import com.ycwl.basic.device.entity.alioss.DeviceAliOssConfig;
import com.ycwl.basic.device.entity.common.FileObject;
import com.ycwl.basic.device.operator.IDeviceStorageOperator;
import com.ycwl.basic.device.operator.helper.CommonPieceGetter;
import lombok.extern.slf4j.Slf4j;

import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
public class AliOssStorageOperator extends CommonPieceGetter<DeviceAliOssConfig> implements IDeviceStorageOperator {
    public AliOssStorageOperator(String configJson) {
        super(configJson);
    }

    private OSS getClient() {
        return new OSSClientBuilder().build(config.getEndpoint(), config.getAccessKeyId(), config.getAccessKeySecret());
    }

    private List<OSSObjectSummary> getOssObjectListByPrefix(String prefix) {
        OSS ossClient = getClient();
        ListObjectsV2Request listObjectsV2Request = new ListObjectsV2Request(config.getBucketName());
        listObjectsV2Request.setPrefix(config.getPrefix() + prefix);
        listObjectsV2Request.setMaxKeys(1000);
        boolean isTruncated = true;
        String continuationToken = null;
        List<OSSObjectSummary> objectList = new ArrayList<>();
        try {
            while (isTruncated) {
                if (continuationToken != null) {
                    listObjectsV2Request.setContinuationToken(continuationToken);
                }

                // 列举文件。
                ListObjectsV2Result result = ossClient.listObjectsV2(listObjectsV2Request);

                objectList.addAll(result.getObjectSummaries());

                isTruncated = result.isTruncated();
                continuationToken = result.getNextContinuationToken();
            }
            return objectList;
        } catch (OSSException e) {
            log.error("获取OSS文件列表失败", e);
        }
        return null;
    }

    private List<FileObject> getOssFileListByPrefix(String prefix) {
        OSS ossClient = getClient();
        List<OSSObjectSummary> objectList = getOssObjectListByPrefix(prefix);
        if (objectList == null) {
            return null;
        }
        return objectList.stream().filter(item -> item.getKey().endsWith(".ts")).map(item -> {
            FileObject object = new FileObject();
            object.setPath(item.getKey());
            URL url = ossClient.generatePresignedUrl(item.getBucketName(), item.getKey(), DateUtil.offsetHour(new Date(), 8), HttpMethod.GET);
            object.setUrl(url.toString());
            object.setNeedDownload(true);
            String[] splitDir = item.getKey().split("/");
            String splitDate = splitDir[splitDir.length - 2];
            String splitName = splitDir[splitDir.length - 1];
            String[] splitExt = splitName.split("\\.", 2);
            String[] splitDt = splitExt[0].split("_", 2);
            String createTime = splitDt[0];
            String endTime = splitDt[1];
            object.setName(splitName);
            object.setEndTime(DateUtil.parse(splitDate+endTime, "yyyyMMddHHmmss"));
            object.setCreateTime(DateUtil.parse(splitDate+createTime, "yyyyMMddHHmmss"));
            return object;
        }).collect(Collectors.toList());
    }

    private boolean removeFilesByPrefix(String prefix) {
        OSS ossClient = getClient();
        List<OSSObjectSummary> objectList = getOssObjectListByPrefix(prefix);
        if (objectList == null) {
            return false;
        }
        int idx = 0;
        int batchSize = 999;
        while (objectList.size() > idx) {
            if (objectList.size() - idx < batchSize) {
                batchSize = objectList.size() - idx;
            }
            List<OSSObjectSummary> subList = objectList.subList(idx, idx + batchSize);
            idx += batchSize;
            DeleteObjectsRequest request = new DeleteObjectsRequest(config.getBucketName());
            request.setKeys(subList.stream().map(OSSObjectSummary::getKey).collect(Collectors.toList()));
            try {
                ossClient.deleteObjects(request);
            } catch (OSSException e) {
                return false;
            }
        }
        ossClient.shutdown();
        return true;
    }

    @Override
    public List<FileObject> getFileListByDtRange(Date startDate, Date endDate) {
        if (startDate == null || endDate == null) {
            return null;
        }
        List<FileObject> fileList = new ArrayList<>();
        if (startDate.after(endDate)) {
            return fileList;
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        calendar.set(Calendar.SECOND, 0);
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd/HHmm");
        while (calendar.getTime().before(endDate)) {
            String prefix = dateFormat.format(calendar.getTime());
            List<FileObject> fileListByPrefix = getOssFileListByPrefix(prefix);
            if (fileListByPrefix == null) {
                return null;
            }
            fileList.addAll(fileListByPrefix);
            calendar.add(Calendar.MINUTE, 1);
        }
        calendar.clear();
        return fileList.stream()
                .sorted(Comparator.comparing(FileObject::getCreateTime))
                .filter(item -> item.getCreateTime().after(startDate))
                .filter(item -> item.getCreateTime().before(endDate))
                .collect(Collectors.toList());
    }

    @Override
    public boolean removeFilesBeforeDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd/");
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        String prefix = dateFormat.format(calendar.getTime());
        return removeFilesByPrefix(prefix);
    }
}
